@@ -44,18 +44,23 @@
 #include <string.h>
 #include <gtk/gtk.h>
 
+/* This is for bonobo_dock_item_set_locked */
+#define BONOBO_UI_INTERNAL
+
 /* Must be before all other gnome includes!! */
 #include "gnome-i18nP.h"
 
-
+#include <gconf/gconf-client.h>
 #include <libgnome/gnome-program.h>
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-config.h>
-#include <libgnome/gnome-preferences.h>
 #include <bonobo/bonobo-dock.h>
 #include "gnome-app-helper.h"
 #include "gnome-uidefs.h"
+#include "gnome-gconf-ui.h"
 #include "gnome-window.h"
+#include "gnome-window-icon.h"
+#include "gnome-ui-init.h"
 
 #include "gnome-app.h"
 
@@ -70,8 +75,6 @@ struct _GnomeAppPrivate
 };
 
 
-static void gnome_app_class_init  (GnomeAppClass *class);
-static void gnome_app_instance_init        (GnomeApp      *app);
 static void gnome_app_destroy     (GtkObject     *object);
 static void gnome_app_finalize    (GObject       *object);
 static void gnome_app_show        (GtkWidget     *widget);
@@ -92,7 +95,7 @@ static void   layout_changed      (GtkWidget     *widget,
 
 /* define _get_type and parent_class */
 GNOME_CLASS_BOILERPLATE (GnomeApp, gnome_app,
-			 GtkWindow, gtk_window, GTK_TYPE_WINDOW)
+			 GtkWindow, GTK_TYPE_WINDOW)
 
 static gchar *
 read_layout_config (GnomeApp *app)
@@ -149,9 +152,10 @@ gnome_app_class_init (GnomeAppClass *class)
 				      g_param_spec_string ("app_id",
 							   _("App ID"),
 							   _("The application ID string"),
-							   NULL,
+							   g_get_prgname (),
 							   (G_PARAM_READABLE |
-							    G_PARAM_WRITABLE)));
+							    G_PARAM_WRITABLE |
+							    G_PARAM_CONSTRUCT)));
 }
 
 static void
@@ -190,19 +194,12 @@ gnome_app_get_property (GObject       *object,
 static void
 gnome_app_instance_init (GnomeApp *app)
 {
-	/* FIXME!!!!!!!
-	const char *str = NULL;
-	GValue value = { 0, };
-	*/
-
+	const char *icons = NULL;
+		
 	app->_priv = NULL;
 	/* XXX: when there is some private stuff enable this
 	app->_priv = g_new0(GnomeAppPrivate, 1);
 	*/
-
-	app->name = NULL;
-	app->prefix = NULL;
-
 	app->accel_group = gtk_accel_group_new ();
 	gtk_window_add_accel_group (GTK_WINDOW (app), app->accel_group);
 	
@@ -210,27 +207,81 @@ gnome_app_instance_init (GnomeApp *app)
 	gtk_container_add (GTK_CONTAINER (app), app->vbox);
 
 	app->dock = bonobo_dock_new ();
+	app->layout = bonobo_dock_layout_new ();
+
 	gtk_box_pack_start (GTK_BOX (app->vbox), app->dock,
 			    TRUE, TRUE, 0);
 
-	gtk_signal_connect (GTK_OBJECT (app->dock),
-			    "layout_changed",
-			    GTK_SIGNAL_FUNC (layout_changed),
-			    (gpointer) app);
-
-	app->layout = bonobo_dock_layout_new ();
-
+	g_signal_connect (app->dock, "layout_changed", G_CALLBACK (layout_changed), app);
+	
 	app->enable_layout_config = TRUE;
 
-	/* FIXME!!!!!!!
-	g_value_init (&value, G_TYPE_STRING);
-	g_object_get_property (G_OBJECT (gnome_program_get ()),
-			       LIBGNOMEUI_PARAM_DEFAULT_ICON, &value);
-	str = g_value_get_string (&value);
-	if (str != NULL)
-		gnome_window_set_icon_from_file (GTK_WINDOW (app), str, FALSE);
-	g_value_unset (&value);
-	*/
+	g_object_get (G_OBJECT (gnome_program_get ()),
+		      LIBGNOMEUI_PARAM_DEFAULT_ICON, &icons,
+		      NULL);
+
+	if (icons && *icons) {
+		char **files = g_strsplit (icons, ";", -1);
+		gnome_window_icon_set_from_file_list (GTK_WINDOW (app),
+						      (const char **)files);
+		g_strfreev (files);
+	}
+}
+
+static void
+remove_notification_cb (BonoboDockItem *item, gpointer data)
+{
+	GConfClient *client;
+	guint notify_id;
+
+	notify_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "gnome-app-gconf-notify-id"));
+
+	if (notify_id == 0)
+		return;
+	
+	client = gconf_client_get_default ();
+	gconf_client_notify_remove (client, notify_id);
+	g_object_unref (client);
+	g_object_set_data (G_OBJECT (item), "gnome-app-gconf-notify-id", NULL);
+	
+}
+
+static void
+dock_item_changed_notify (GConfClient* client,
+			  guint cnxn_id,
+			  GConfEntry *entry,
+			  gpointer user_data)
+{
+	BonoboDockItem *item;
+	guint notify_id;
+	
+	item = BONOBO_DOCK_ITEM (user_data);
+	notify_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "gnome-app-gconf-notify-id"));
+
+	if (notify_id == cnxn_id &&
+	    entry->value->type == GCONF_VALUE_BOOL) {
+		gboolean detachable;
+
+		detachable = gconf_value_get_bool (entry->value);
+		
+		/* Update */
+		bonobo_dock_item_set_locked (item, !detachable);
+	}
+}
+
+static void
+setup_notification_for_dock_item (BonoboDockItem *item, const char *key_name)
+{
+	guint notify_id;
+	GConfClient *client;
+
+	client = gconf_client_get_default ();
+	notify_id = gconf_client_notify_add (client, key_name,
+					     dock_item_changed_notify,
+					     item, NULL, NULL);
+
+	g_object_set_data (G_OBJECT (item), "gnome-app-gconf-notify-id", GINT_TO_POINTER (notify_id));
+	g_signal_connect (item, "destroy", G_CALLBACK (remove_notification_cb), NULL);
 }
 
 static void
@@ -257,14 +308,14 @@ gnome_app_show (GtkWidget *widget)
 		if (app->enable_layout_config)
 			write_layout_config (app, app->layout);
 
-		gtk_object_unref (GTK_OBJECT (app->layout));
+		g_object_unref (G_OBJECT (app->layout));
 		app->layout = NULL;
 	}
 			
 	gtk_widget_show (app->vbox);
 	gtk_widget_show (app->dock);
 
-	GNOME_CALL_PARENT_HANDLER (GTK_WIDGET_CLASS, show, (widget));
+	GNOME_CALL_PARENT (GTK_WIDGET_CLASS, show, (widget));
 }
 
 static void
@@ -282,22 +333,21 @@ layout_changed (GtkWidget *w, gpointer data)
 
 		layout = bonobo_dock_get_layout (BONOBO_DOCK (app->dock));
 		write_layout_config (app, layout);
-		gtk_object_unref (GTK_OBJECT (layout));
+		g_object_unref (G_OBJECT (layout));
 	}
 }
 
 /**
- * gnome_app_new
- * @appname: Name of program, using in file names and paths.
+ * gnome_app_new:
+ * @appname: Name of program, used in file names and paths.
  * @title: Window title for application.
  *
  * Description:
- * Create a new (empty) application window.  You must specify the
- * application's name (used internally as an identifier).
- * @title can be left as NULL, in which case the window's title will
- * not be set.
+ * Create a new (empty) application window. You must specify the @appname (used
+ * internally as an identifier).  The @title param can be left as NULL, in
+ * which case the window's title will not be set.
  *
- * Returns: Pointer to new GNOME app object.
+ * Returns: Pointer to new #GnomeApp object.
  **/
 
 GtkWidget *
@@ -310,10 +360,8 @@ gnome_app_new (const gchar *appname, const gchar *title)
 	if (title != NULL) {
 		app = g_object_new (GNOME_TYPE_APP,
 				    "app_id", appname,
+				    "title", title,
 				    NULL);
-		gtk_object_set (GTK_OBJECT (app),
-				"title", title,
-				NULL);
 	} else {
 		app = g_object_new (GNOME_TYPE_APP,
 				    "app_id", appname,
@@ -323,8 +371,8 @@ gnome_app_new (const gchar *appname, const gchar *title)
 }
 
 /**
- * gnome_app_construct
- * @app: Pointer to newly-created GNOME app object.
+ * gnome_app_construct:
+ * @app: A newly created #GnomeApp object.
  * @appname: Name of program, using in file names and paths.
  * @title: Window title for application.
  *
@@ -340,10 +388,8 @@ gnome_app_construct (GnomeApp *app, const gchar *appname, const gchar *title)
 	if (title != NULL) {
 		g_object_set (G_OBJECT (app),
 			      "app_id", appname,
+			      "title", title,
 			      NULL);
-		gtk_object_set (GTK_OBJECT (app),
-				"title", title,
-				NULL);
 	} else {
 		g_object_set (G_OBJECT (app),
 			      "app_id", appname,
@@ -363,22 +409,17 @@ gnome_app_destroy (GtkObject *object)
 
 	app = GNOME_APP (object);
 
-	g_free (app->name);
-	app->name = NULL;
-	g_free (app->prefix);
-	app->prefix = NULL;
-
 	if (app->accel_group != NULL) {
-		gtk_accel_group_unref (app->accel_group);
+		g_object_unref (G_OBJECT (app->accel_group));
 		app->accel_group = NULL;
 	}
 
 	if (app->layout != NULL) {
-		gtk_object_unref (GTK_OBJECT (app->layout));
+		g_object_unref (app->layout);
 		app->layout = NULL;
 	}
 
-	GNOME_CALL_PARENT_HANDLER (GTK_OBJECT_CLASS, destroy, (object));
+	GNOME_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 static void
@@ -391,11 +432,16 @@ gnome_app_finalize (GObject *object)
 
 	app = GNOME_APP (object);
 
+	g_free (app->name);
+	app->name = NULL;
+	g_free (app->prefix);
+	app->prefix = NULL;
+
 	/* Free private data */
 	g_free (app->_priv);
 	app->_priv = NULL;
 
-	GNOME_CALL_PARENT_HANDLER (G_OBJECT_CLASS, finalize, (object));
+	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 /* Callback for an app's contents' parent_set signal.  We set the app->contents
@@ -410,20 +456,18 @@ contents_parent_set (GtkWidget *widget, GtkWidget *previous_parent, gpointer dat
 
 	g_assert (previous_parent == app->dock);
 
-	gtk_signal_disconnect_by_func (GTK_OBJECT (widget),
-				       GTK_SIGNAL_FUNC (contents_parent_set),
-				       data);
+	g_signal_handlers_disconnect_by_func (widget, contents_parent_set, data);
 
 	app->contents = NULL;
 }
 
 /**
- * gnome_app_set_contents
- * @app: Pointer to GNOME app object.
+ * gnome_app_set_contents:
+ * @app: A #GnomeApp instance.
  * @contents: Widget to be application content area.
  *
  * Description:
- * Sets the content area of the GNOME app's main window.
+ * Sets the content area of the main window of @app.
  **/
 void
 gnome_app_set_contents (GnomeApp *app, GtkWidget *contents)
@@ -443,18 +487,18 @@ gnome_app_set_contents (GnomeApp *app, GtkWidget *contents)
 
 	if (new_contents == contents && new_contents != app->contents) {
 		gtk_widget_show (new_contents);
-		gtk_signal_connect (GTK_OBJECT (new_contents), "parent_set",
-				    GTK_SIGNAL_FUNC (contents_parent_set),
-				    app);
+		g_signal_connect (G_OBJECT (new_contents), "parent_set",
+				  G_CALLBACK (contents_parent_set),
+				  app);
 
 		app->contents = new_contents;
 	}
 }
 
 /**
- * gnome_app_set_menus
- * @app: Pointer to GNOME app object.
- * @menubar: Menu bar widget for main app window.
+ * gnome_app_set_menus:
+ * @app: A #GnomeApp instance.
+ * @menubar: Menu bar widget for main application window.
  *
  * Description:
  * Sets the menu bar of the application window.
@@ -476,35 +520,18 @@ gnome_app_set_menus (GnomeApp *app, GtkMenuBar *menubar)
 	behavior = (BONOBO_DOCK_ITEM_BEH_EXCLUSIVE
 		    | BONOBO_DOCK_ITEM_BEH_NEVER_VERTICAL);
 	
-	if (!gnome_preferences_get_menubar_detachable())
+	if ( ! gnome_gconf_get_bool ("/desktop/gnome/interface/menubar_detachable"))
 		behavior |= BONOBO_DOCK_ITEM_BEH_LOCKED;
 
 	dock_item = bonobo_dock_item_new (GNOME_APP_MENUBAR_NAME,
 					 behavior);
+	setup_notification_for_dock_item (BONOBO_DOCK_ITEM (dock_item), "/desktop/gnome/interface/menubar_detachable");
 	gtk_container_add (GTK_CONTAINER (dock_item), GTK_WIDGET (menubar));
 
 	app->menubar = GTK_WIDGET (menubar);
-
-	/* To have menubar relief agree with the toolbar (and have the relief
-	 * outside of smaller handles), substitute the dock item's relief for
-	 * the menubar's relief, but don't change the size of the menubar in
-	 * the process. */
-#ifdef FIXME
-	gtk_menu_bar_set_shadow_type (GTK_MENU_BAR (app->menubar),
-				      GTK_SHADOW_NONE);
-#endif
-	if (gnome_preferences_get_menubar_relief ()) {
-		guint border_width;
-		
-		gtk_container_set_border_width (GTK_CONTAINER (dock_item), 2);
-		border_width = GTK_CONTAINER (app->menubar)->border_width;
-		if (border_width >= 2)
-			border_width -= 2;
-		gtk_container_set_border_width (GTK_CONTAINER (app->menubar),
-						border_width);
-	} else
-		bonobo_dock_item_set_shadow_type (BONOBO_DOCK_ITEM (dock_item),
-						 GTK_SHADOW_NONE);
+	
+	bonobo_dock_item_set_shadow_type (BONOBO_DOCK_ITEM (dock_item),
+					  GTK_SHADOW_NONE);
 
 	if (app->layout != NULL)
 		bonobo_dock_layout_add_item (app->layout,
@@ -520,15 +547,15 @@ gnome_app_set_menus (GnomeApp *app, GtkMenuBar *menubar)
 	gtk_widget_show (GTK_WIDGET (menubar));
 	gtk_widget_show (GTK_WIDGET (dock_item));
 
-	ag = gtk_object_get_data(GTK_OBJECT(app), "GtkAccelGroup");
+	ag = g_object_get_data(G_OBJECT(app), "GtkAccelGroup");
 	if (ag && !g_slist_find(gtk_accel_groups_from_object (G_OBJECT (app)), ag))
 	        gtk_window_add_accel_group(GTK_WINDOW(app), ag);
 }
 
 
 /**
- * gnome_app_set_statusbar
- * @app: Pointer to GNOME app object
+ * gnome_app_set_statusbar:
+ * @app: A #GnomeApp instance
  * @statusbar: Statusbar widget for main app window
  *
  * Description:
@@ -558,8 +585,8 @@ gnome_app_set_statusbar (GnomeApp *app,
 }
 
 /**
- * gnome_app_set_statusbar_custom
- * @app: Pointer to GNOME app object
+ * gnome_app_set_statusbar_custom:
+ * @app: A #GnomeApp instance
  * @container: container widget containing the statusbar
  * @statusbar: Statusbar widget for main app window
  *
@@ -588,7 +615,7 @@ gnome_app_set_statusbar_custom (GnomeApp *app,
 
 /**
  * gnome_app_add_toolbar:
- * @app: A &GnomeApp widget
+ * @app: A #GnomeApp widget
  * @toolbar: Toolbar to be added to @app's dock
  * @name: Name for the dock item that will contain @toolbar
  * @behavior: Behavior for the new dock item
@@ -598,7 +625,7 @@ gnome_app_set_statusbar_custom (GnomeApp *app,
  * @offset: Offset from the previous dock item in the band; if there is
  * no previous item, offset from the beginning of the band.
  *
- * Create a new &BonoboDockItem widget containing @toolbar, and add it
+ * Create a new #BonoboDockItem widget containing @toolbar, and add it
  * to @app's dock with the specified layout information.  Notice that,
  * if automatic layout configuration is enabled, the layout is
  * overridden by the saved configuration, if any.
@@ -622,8 +649,8 @@ gnome_app_add_toolbar (GnomeApp *app,
 	g_return_if_fail(toolbar != NULL);
 
 	dock_item = bonobo_dock_item_new (name, behavior);
-
-	gtk_container_set_border_width (GTK_CONTAINER (toolbar), 1);
+	setup_notification_for_dock_item (BONOBO_DOCK_ITEM (dock_item), "/desktop/gnome/interface/toolbar_detachable");
+	
 	gtk_container_add (GTK_CONTAINER (dock_item), GTK_WIDGET (toolbar));
 
 	if(app->layout)
@@ -648,14 +675,14 @@ gnome_app_add_toolbar (GnomeApp *app,
 	gtk_widget_show (GTK_WIDGET (toolbar));
 	gtk_widget_show (GTK_WIDGET (dock_item));
 
-	ag = gtk_object_get_data(GTK_OBJECT(app), "GtkAccelGroup");
+	ag = g_object_get_data(G_OBJECT(app), "GtkAccelGroup");
 	if (ag && !g_slist_find(gtk_accel_groups_from_object (G_OBJECT (app)), ag))
 	        gtk_window_add_accel_group(GTK_WINDOW(app), ag);
 }
 
 /**
- * gnome_app_set_toolbar
- * @app: Pointer to GNOME app object.
+ * gnome_app_set_toolbar:
+ * @app: A #GnomeApp instance.
  * @toolbar: Toolbar widget for main app window.
  *
  * Description:
@@ -673,7 +700,7 @@ gnome_app_set_toolbar (GnomeApp *app,
 	   requirement.  We only do this for backwards compatibility.  */
 	behavior = BONOBO_DOCK_ITEM_BEH_EXCLUSIVE;
 	
-	if (!gnome_preferences_get_toolbar_detachable())
+	if ( ! gnome_gconf_get_bool ("/desktop/gnome/interface/toolbar_detachable"))
 		behavior |= BONOBO_DOCK_ITEM_BEH_LOCKED;
 	
 	gnome_app_add_toolbar (app, toolbar,
@@ -686,11 +713,11 @@ gnome_app_set_toolbar (GnomeApp *app,
 
 /**
  * gnome_app_add_dock_item:
- * @app: A &GnomeApp widget
+ * @app: A #GnomeApp widget.
  * @item: Dock item to be added to @app's dock.
- * @placement: Placement for the dock item
- * @band_num: Number of the band where the dock item should be placed
- * @band_position: Position of the dock item in band @band_num
+ * @placement: Placement for the dock item.
+ * @band_num: Number of the band where the dock item should be placed.
+ * @band_position: Position of the dock item in band @band_num.
  * @offset: Offset from the previous dock item in the band; if there is
  * no previous item, offset from the beginning of the band.
  * 
@@ -722,29 +749,27 @@ gnome_app_add_dock_item (GnomeApp *app,
 				     offset,
 				     FALSE);
 
-	gtk_signal_emit_by_name (GTK_OBJECT (app->dock),
-				 "layout_changed",
-				 (gpointer) app);
+	g_signal_emit_by_name (app->dock, "layout_changed", app);
 }
 
 /**
  * gnome_app_add_docked:
- * @app: A &GnomeApp widget
- * @widget: Widget to be added to the &GnomeApp
- * @name: Name for the new dock item
- * @behavior: Behavior for the new dock item
- * @placement: Placement for the new dock item
- * @band_num: Number of the band where the dock item should be placed
- * @band_position: Position of the new dock item in band @band_num
+ * @app: A #GnomeApp widget.
+ * @widget: Widget to be added to the #GnomeApp.
+ * @name: Name for the new dock item.
+ * @behavior: Behavior for the new dock item.
+ * @placement: Placement for the new dock item.
+ * @band_num: Number of the band where the dock item should be placed.
+ * @band_position: Position of the new dock item in band @band_num.
  * @offset: Offset from the previous dock item in the band; if there is
  * no previous item, offset from the beginning of the band.
- *
- * Returns: The dock item used to contain the widget.
  * 
  * Add @widget as a dock item according to the specified layout
  * information.  Notice that, if automatic layout configuration is
  * enabled, the layout is overridden by the saved configuration, if
  * any.
+ *
+ * Returns: The dock item used to contain the widget.
  **/
 GtkWidget *
 gnome_app_add_docked (GnomeApp *app,
@@ -759,6 +784,7 @@ gnome_app_add_docked (GnomeApp *app,
 	GtkWidget *item;
 
 	item = bonobo_dock_item_new (name, behavior);
+	setup_notification_for_dock_item (BONOBO_DOCK_ITEM (item), "/desktop/gnome/interface/toolbar_detachable");
 	gtk_container_add (GTK_CONTAINER (item), widget);
 	gnome_app_add_dock_item (app, BONOBO_DOCK_ITEM (item),
 				 placement, band_num, band_position, offset);
@@ -768,9 +794,9 @@ gnome_app_add_docked (GnomeApp *app,
 
 /**
  * gnome_app_enable_layout_config:
- * @app: A &GnomeApp widget
+ * @app: A #GnomeApp widget.
  * @enable: Boolean specifying whether automatic configuration saving
- * is enabled
+ * is enabled.
  * 
  * Specify whether @app should automatically save the dock's
  * layout configuration via gnome-config whenever it changes or not.
@@ -783,12 +809,12 @@ gnome_app_enable_layout_config (GnomeApp *app, gboolean enable)
 
 /**
  * gnome_app_get_dock_item_by_name:
- * @app: A &GnomeApp widget
- * @name: Name of the dock item to retrieve
+ * @app: A #GnomeApp widget.
+ * @name: Name of the dock item to retrieve.
  * 
  * Retrieve the dock item whose name matches @name.
  * 
- * Return value: The retrieved dock item.
+ * Returns: The retrieved dock item.
  **/
 BonoboDockItem *
 gnome_app_get_dock_item_by_name (GnomeApp *app,
@@ -815,11 +841,11 @@ gnome_app_get_dock_item_by_name (GnomeApp *app,
 
 /**
  * gnome_app_get_dock:
- * @app: A &GnomeApp widget
+ * @app: A #GnomeApp widget
  * 
- * Retrieves the &BonoboDock widget contained in the &GnomeApp.
+ * Retrieves the #BonoboDock widget contained in @app.
  * 
- * Returns: The &BonoboDock widget.
+ * Returns: The relevant #BonoboDock widget.
  **/
 BonoboDock *
 gnome_app_get_dock (GnomeApp *app)
