@@ -176,8 +176,6 @@ present_authentication_dialog_nonblocking (CallbackInfo *info)
 		gtk_window_set_modal (GTK_WINDOW (dialog), FALSE);
 	}
 
-	gtk_window_set_modal (GTK_WINDOW (dialog), FALSE);
-
 	g_signal_connect (dialog, "response", 
 			  G_CALLBACK (authentication_dialog_button_clicked), info);
 
@@ -210,6 +208,8 @@ vfs_async_authentication_callback (gconstpointer in, size_t in_size,
 	g_return_if_fail (in != NULL);
 	g_return_if_fail (out != NULL);
 
+	GDK_THREADS_ENTER ();
+	
 	in_real = (GnomeVFSModuleCallbackAuthenticationIn *)in;
 	out_real = (GnomeVFSModuleCallbackAuthenticationOut *)out;
 
@@ -228,6 +228,8 @@ vfs_async_authentication_callback (gconstpointer in, size_t in_size,
 	present_authentication_dialog_nonblocking (info);
 
 	DEBUG_MSG (("-%s\n", G_GNUC_FUNCTION));
+
+	GDK_THREADS_LEAVE ();
 }
 
 static void /* GnomeVFSModuleCallback */
@@ -245,6 +247,8 @@ vfs_authentication_callback (gconstpointer in, size_t in_size,
 	g_return_if_fail (in != NULL);
 	g_return_if_fail (out != NULL);
 
+	GDK_THREADS_ENTER ();
+	
 	in_real = (GnomeVFSModuleCallbackAuthenticationIn *)in;
 	out_real = (GnomeVFSModuleCallbackAuthenticationOut *)out;
 
@@ -255,6 +259,8 @@ vfs_authentication_callback (gconstpointer in, size_t in_size,
 	present_authentication_dialog_blocking (is_proxy_authentication, in_real, out_real);
 
 	DEBUG_MSG (("-%s\n", G_GNUC_FUNCTION));
+
+	GDK_THREADS_LEAVE ();
 }
 
 static GnomePasswordDialog *
@@ -279,10 +285,11 @@ construct_full_password_dialog (const GnomeVFSModuleCallbackFullAuthenticationIn
 		g_string_append_printf (name, "/%s", in_args->object);
 	}
 	if (in_args->domain != NULL) {
-		g_string_append_printf (name, _(" domain %s"), in_args->domain);
+		message = g_strdup_printf (_("You must log in to access %s domain %s\n"), name->str, in_args->domain);
+	} else {
+		message = g_strdup_printf (_("You must log in to access %s\n"), name->str);
 	}
 
-	message = g_strdup_printf (_("You must log in to access %s\n"), name->str);
 	g_string_free (name, TRUE);
 		
 	dialog = GNOME_PASSWORD_DIALOG (gnome_password_dialog_new (
@@ -300,7 +307,8 @@ construct_full_password_dialog (const GnomeVFSModuleCallbackFullAuthenticationIn
 					       in_args->flags & GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION_NEED_DOMAIN);
 	gnome_password_dialog_set_show_password (dialog, 
 						 in_args->flags & GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION_NEED_PASSWORD);
-	gnome_password_dialog_set_show_remember (dialog, gnome_keyring_is_available ());
+	gnome_password_dialog_set_show_remember (dialog, gnome_keyring_is_available () && 
+							 in_args->flags & GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION_SAVING_SUPPORTED);
 
 	return dialog;
 }
@@ -437,6 +445,8 @@ vfs_async_full_authentication_callback (gconstpointer in, size_t in_size,
 	g_return_if_fail (in != NULL);
 	g_return_if_fail (out != NULL);
 
+	GDK_THREADS_ENTER ();
+	
 	in_real = (GnomeVFSModuleCallbackFullAuthenticationIn *)in;
 	out_real = (GnomeVFSModuleCallbackFullAuthenticationOut *)out;
 
@@ -452,6 +462,8 @@ vfs_async_full_authentication_callback (gconstpointer in, size_t in_size,
 	present_full_authentication_dialog_nonblocking (info);
 
 	DEBUG_MSG (("-%s\n", G_GNUC_FUNCTION));
+
+	GDK_THREADS_LEAVE ();
 }
 
 static void /* GnomeVFSModuleCallback */
@@ -468,6 +480,8 @@ vfs_full_authentication_callback (gconstpointer in, size_t in_size,
 	g_return_if_fail (in != NULL);
 	g_return_if_fail (out != NULL);
 
+	GDK_THREADS_ENTER ();
+	
 	in_real = (GnomeVFSModuleCallbackFullAuthenticationIn *)in;
 	out_real = (GnomeVFSModuleCallbackFullAuthenticationOut *)out;
 
@@ -476,6 +490,8 @@ vfs_full_authentication_callback (gconstpointer in, size_t in_size,
 	present_full_authentication_dialog_blocking (in_real, out_real);
 
 	DEBUG_MSG (("-%s\n", G_GNUC_FUNCTION));
+
+	GDK_THREADS_LEAVE ();
 }
 
 
@@ -705,6 +721,11 @@ vfs_save_authentication_callback (gconstpointer in, size_t in_size,
 	DEBUG_MSG (("-%s\n", G_GNUC_FUNCTION));
 }
 
+/* If you call this, and you use threads with gtk+, you must never
+ * hold the gdk lock while doing synchronous gnome-vfs calls. Otherwise
+ * an authentication callback presenting a dialog could try to grab the
+ * already held gdk lock, causing a deadlock.
+ */
 void
 gnome_authentication_manager_init (void)
 {
